@@ -7,7 +7,7 @@ from typing import Optional, Union
 
 from ..yhteys import AsynkroninenYhteys
 from ..sanoma import RestSanoma
-from ..tyokalut import luokkamaare
+from ..tyokalut import ei_syotetty, luokkamaare, periyta, Valinnainen
 
 
 class RajapintaMeta(type):
@@ -58,30 +58,41 @@ class Rajapinta(metaclass=RajapintaMeta):
 
   @dataclass
   class Syote(RestSanoma):
-    @luokkamaare
-    def Paivitys(cls):
-      '''
-      Tietorakenne olemassaolevan tietueen päivittämiseen.
+    ''' Lähtevän datan tietorakenne. '''
 
-      Oletuksena käytetään samaa Syötettä kuin uudelle tietueelle.
-      '''
-      return cls
-      # def Paivitys
-    # class Syote
+  @luokkamaare
+  def Paivitys(cls):
+    '''
+    Tietorakenne olemassaolevan tietueen päivittämiseen.
 
-  @dataclass
+    Oletuksena käytetään samaa Syötettä kuin uudelle tietueelle.
+    '''
+    return cls.Syote
+    # def Paivitys
+
+  @dataclass(kw_only=True)
   class Tuloste(RestSanoma):
-    pass
+    ''' Saapuvan datan tietorakenne. '''
 
   class Meta:
+    ''' Rajapinnan metatiedot. '''
     # Tiedonvaihtoon käytetty URL, esim. /api/kioski/
     rajapinta: str
-    rajapinta_pk: Optional[str] = None  # /api/kioski/%(pk)s/
+
+    # Tietuekohtaiseen tiedonvaihtoon käytetty URL,
+    # oletuksena {rajapinta}/%(pk)s.
+    rajapinta_pk: Optional[str] = None
+
+    # Primääriavain tietueen kentissä,
+    pk: str = 'id'
+
+    # class Meta
 
   def __call__(self, *args, **kwargs):
     return self.Syote(*args, **kwargs)
 
   def _tulkitse_saapuva(self, saapuva):
+    ''' Tulkitse saapuvan datan sisältämä sanoma. '''
     if not isinstance(saapuva, Mapping):
       raise TypeError(
         f'Noudettu data ei ole kuvaus: {type(saapuva)!r}!'
@@ -90,6 +101,7 @@ class Rajapinta(metaclass=RajapintaMeta):
     # def _tulkitse_saapuva
 
   def _tulkitse_lahteva(self, lahteva):
+    ''' Muodosta lähtevä data sanomalle. '''
     return lahteva.lahteva()
     # def _tulkitse_lahteva
 
@@ -115,6 +127,20 @@ class Rajapinta(metaclass=RajapintaMeta):
       return [self._tulkitse_saapuva(d) for d in data]
     # async def nouda
 
+  async def otsakkeet(self, **params):
+    return await self.yhteys.nouda_otsakkeet(
+      self.Meta.rajapinta,
+      params=params,
+    )
+    # async def otsakkeet
+
+  async def meta(self, **params):
+    return await self.yhteys.nouda_meta(
+      self.Meta.rajapinta,
+      params=params,
+    )
+    # async def meta
+
   async def lisaa(
     self,
     data: Optional[Syote] = None,
@@ -146,7 +172,7 @@ class Rajapinta(metaclass=RajapintaMeta):
         'Anna joko syöte tai `kwargs`.'
       )
     elif kwargs:
-      data = self.Syote.Paivitys(**kwargs)
+      data = self.Paivitys(**kwargs)
     return self._tulkitse_saapuva(
       await self.yhteys.muuta_data(
         self.Meta.rajapinta_pk % {'pk': pk},
